"""
Basically, this should replace the runner functionality from trigger so that
we can more easily debug what's going on without sending stuff out
"""

import logging

import resultsdb_api

from . import config
from . import exceptions as exc


_log = logging.getLogger(__name__)


class BaseSender(object):
    def send(self, result):
        raise NotImplementedError


class ResultsdbSender(BaseSender):
    def __init__(self, rdb_connection=None):
        if rdb_connection is None:
            self.rdb_connection = resultsdb_api.ResultsDBapi(
                config.resultsdb_url
            )
        else:
            self.rdb_connection = rdb_connection

    def _check_testcase_exists(self, testcase):
        try:
            self.rdb_connection.get_testcase(testcase)
        except resultsdb_api.ResultsDBapiException:
            raise exc.SenderError("Testcase was not found")

    def _create_testcase(self, testcase):
        self.rdb_connection.create_testcase(testcase)

    def _translate_outcome(self, message):
        testcase = message["topic"]

        if testcase.endswith(".error"):
            return "FAILED"
        if testcase.endswith(".queued"):
            return "QUEUED"
        if testcase.endswith(".running"):
            return "RUNNING"

        outcome_mapping = {
            "SUCCESS": "PASSED",
            "FAILURE": "FAILED",
            "UNSTABLE": "FAILED",
            # outcome values defined on message version 0.2.1
            # https://pagure.io/fedora-ci/messages/tree/6715da742c6669fa725f615872291c58f82d3718
            "PASSED": "PASSED",
            "FAILED": "FAILED",
            "NEEDS_INSPECTION": "FAILED",
        }

        if "status" not in message:
            return "FAILED"

        outcome = message["status"].upper()

        if outcome in outcome_mapping:
            return outcome_mapping[outcome]
        else:
            return "FAILED"

    # topic is the testcase
    # add all data as kv pairs
    def send(self, result):
        # Old version of CI messages didn't define namespace, type or category
        # topic was used for test case name
        if (
            "namespace" in result
            and "test_type" in result
            and "category" in result
        ):
            testcase_name = ".".join(
                [result["namespace"], result["test_type"], result["category"]]
            )
        else:
            testcase_name = result["topic"]

        # make sure that the testcase exists, if not, create it
        try:
            self._check_testcase_exists(testcase_name)
        except exc.SenderError:
            self._create_testcase(testcase_name)

        # translate the outcome from CI to resultsdb-speak
        rdb_outcome = self._translate_outcome(result)

        # construct the data that is to be sent
        ref_url = result["build_url"]

        # send the result to resultsdb
        try:
            self.rdb_connection.create_result(
                outcome=rdb_outcome,
                testcase=testcase_name,
                group=None,
                note=None,
                ref_url=ref_url,
                **result
            )
        except resultsdb_api.ResultsDBapiException:
            _log.exception("An error occured while uploading to resultsdb")


class StreamSender(BaseSender):
    def send(self, result):
        print("StreamSender\n------------")
        print(result)


_senders = {"ResultsdbSender": ResultsdbSender, "StreamSender": StreamSender}


def get_sender(sender_type):
    return _senders[sender_type]
